# vim:syn=make:
#**********************************************************************
#
# Xilinx Place and Route FPGA Makefile
#
#**********************************************************************
# <Design Name>
#
# This is the Xilinx place and route makefile, which builds the 
# FPGA design using either the Xilinx XST synthesizer or the 
# Synopsys FPGA synthesizer (a.k.a., Synplify Pro) tools. 
# The synthesis build is executed in a different level of the design
# hierarchy. The resulting EDIF files are copied to the ../par/run/
# directory and used with the Xilinx back-end tools.
# This makefile is copied from the ../par/bin/ directory to the 
# ../par/run/ directory and executed from a top-level place and route
# makefile. All results from the build are stored in the log, out, 
# and run directories.  This script is dedicated to the Xilinx ISE toolset. 
#
#**********************************************************************

# Project Name:
PROJNAME  := myfpga

# Directory Variables:
PROJDIR   := ..
COMMONDIR := ../../../common/bin
SRCDIR    := $(PROJDIR)/src
BINDIR    := $(PROJDIR)/bin
LOGDIR    := $(PROJDIR)/log
OUTDIR    := $(PROJDIR)/out
RUNDIR    := $(PROJDIR)/run
SYNDIR    := $(PROJDIR)/../syn
CODEDIR   := ../../src
CFGDIR    := ../../cfg

# Translate File Variables:
UCF       := $(SRCDIR)/$(PROJNAME).ucf
SYNTHFILE := $(RUNDIR)/$(PROJNAME).edf
TRANOPTS  := -p $(CHIP) -intstyle silent -uc $(UCF) -sd "../run/" $(SYNTHFILE) $(PROJNAME).ngd

# MAP File Variables:
MAPOPTS   := -p $(CHIP) –w -logic_opt off -ol high -t 1 -xt 0 -register_duplication off -r 4 -global_opt off -mt 2 –detail \
-ir off -pr b -lc off -power off -o map.ncd

# PAR File Variables:
PAROPTS   := -w -ol high -mt 2

# Bitgen File Variables:
BITOPTS   := -w -m -g DebugBitstream:No -g Binary:no -g CRC:Enable -g ConfigRate:2 -g CclkPin:PullUp -g M0Pin:PullNone \
-g M1Pin:PullNone -g M2Pin:PullNone -g ProgPin:PullUp -g InitPin:Pullnone -g CsPin:Pullnone -g DinPin:Pullnone -g BusyPin:Pullnone \
-g RdWrPin:Pullnone -g HswapenPin:PullUp -g TckPin:PullNone -g TdiPin:PullNone -g TdoPin:PullNone -g TmsPin:PullNone \
-g Disable_JTAG:No -g UnusedPin:PullNone -g UserID:0xDEADBEEF -g ConfigFallback:Enable -g BPI_page_size:1 \
-g OverTempPowerDown:Disable -g next_config_addr:None -g JTAG_SysMon:Enable -g DCIUpdateMode:Quiet -g StartUpClk:CClk \
-g DONE_cycle:4 -g GTS_cycle:5 -g GWE_cycle:6 -g Match_cycle:NoWait -g Security:Level2 -g DonePipe:No -g DriveDone:Yes \
-g Encrypt:No

# PromGen File Variables:
PROMOPTS  := -u 0x0 $(PROJNAME).bit -p mcs -o $(PROJNAME).mcs -s 16384 -spi -w

# Trace Variables:
TRACEOPTS := -e 3 -l 3 $(PROJNAME).ncd $(PROJNAME).pcf -xml $(LOGDIR)/$(PROJNAME) -o $(PROJNAME).twr

# SDF Variables:
SDFOPTS   := -w -ofmt verilog -aka -fn -pcf $(PROJNAME).pcf -s 2 -sim -tb -ism -ne -sdf_anno true $(PROJNAME).ncd


# Environment Variables:
export SYN_TCL_HOOKS=../bin/synhooks.tcl
export XIL_PAR_DESIGN_CHECK_VERBOSE=1

default:
	@echo "** Xilinx Place and Route Build ************************************"
	@echo "targets:"
	@echo "  make synthesize     - synthesize chip"
	@echo "  make translate      - translate chip"
	@echo "  make map            - map chip"
	@echo "  make par            - par chip"
	@echo "  make bit            - generate bit file"
	@echo "  make prom           - generate prom file"
	@echo "  make trace          - run timing analyzer"
	@echo "  make sdf            - generate post place & route files"
	@echo "  make download       - program entire JTAG chain."
	@echo "  make all            - run all make targets"
	@echo "  make clean          - clean current build folder"
	@echo "********************************************************************"

synthesize: 
		@echo "********************************************************************"
		@echo "    Launch Synthesizer"
		@echo "********************************************************************"
		cd $(SYNDIR)/bin; make synthesize;

translate : $(PROJNAME).ngd

$(PROJNAME).ngd : $(SYNTHFILE) $(UCF)
		@echo "********************************************************************"
		@echo "    Launch NGDBUILD"
		@echo "********************************************************************"
		ngdbuild -f $(TRANOPTS) $(SYNTHFILE) $(PROJNAME).ngd
		mv $(RUNDIR)/$(PROJNAME).bld $(LOGDIR)/
		mv $(RUNDIR)/netlist.lst $(LOGDIR)/
		cp -f $(RUNDIR)/$(PROJNAME).log $(LOGDIR)/

map : map.ncd

map.ncd : $(PROJNAME).ngd
		@echo "********************************************************************"
		@echo "    Launch MAP"
		@echo "********************************************************************"
		map  -f $(MAPOPTS) $(PROJNAME).ngd $(PROJNAME).pcf
		mv $(RUNDIR)/map.mrp $(LOGDIR)/
		mv $(RUNDIR)/map.map $(LOGDIR)/
		mv $(RUNDIR)/$(PROJNAME)*.xml $(LOGDIR)/
		cp -f $(RUNDIR)/$(PROJNAME).log $(LOGDIR)/

par : $(PROJNAME).ncd

$(PROJNAME).ncd : map.ncd
		@echo "********************************************************************"
		@echo "    Launch PAR"
		@echo "********************************************************************"
		par -f $(PAROPTS) map.ncd $(PROJNAME).ncd $(PROJNAME).pcf
		mv $(RUNDIR)/$(PROJNAME).unroutes $(LOGDIR)/
		mv $(RUNDIR)/$(PROJNAME).pad $(LOGDIR)/
		mv $(RUNDIR)/$(PROJNAME).par $(LOGDIR)/
		mv $(RUNDIR)/$(PROJNAME).xpi $(LOGDIR)/
		mv $(RUNDIR)/$(PROJNAME)_pad.csv $(LOGDIR)/
		mv $(RUNDIR)/$(PROJNAME)_pad.txt $(LOGDIR)/
		cp -f $(RUNDIR)/$(PROJNAME).log $(LOGDIR)/

bit : $(PROJNAME).bit

$(PROJNAME).bit : $(INDEX) $(PROJNAME).ncd
		@echo "********************************************************************"
		@echo "    Launch BITGEN"
		@echo "********************************************************************"
		bitgen $(PROJNAME).ncd -f $(BITOPTS)
		cp $(PROJNAME).bit $(OUTDIR)/$(PROJNAME).bit
		cp $(PROJNAME).msk $(OUTDIR)/$(PROJNAME).msk
		cp $(PROJNAME).bit $(CFGDIR)/
		cp $(PROJNAME).msk $(CFGDIR)/
		mv $(RUNDIR)/$(PROJNAME).bgn $(LOGDIR)/
		mv $(RUNDIR)/$(PROJNAME).drc $(LOGDIR)/
		mv $(RUNDIR)/$(PROJNAME)*.xml $(LOGDIR)/
		
prom : $(PROJNAME).mcs

$(PROJNAME).mcs : $(PROJNAME).bit
		@echo "********************************************************************"
		@echo "    Launch PROMGEN for SPI PROM"
		@echo "********************************************************************"
		promgen -f $(PROMOPTS)
		cp $(PROJNAME).mcs $(OUTDIR)/$(PROJNAME).mcs
		cp $(PROJNAME).mcs $(CFGDIR)/$(PROJNAME).mcs
		cp $(PROJNAME).cfi $(CFGDIR)/$(PROJNAME).cfi
		mv $(PROJNAME).prm $(LOGDIR)/$(PROJNAME)_SPI.prm

trace : $(PROJNAME).twr

$(PROJNAME).twr : $(PROJNAME).ncd
		@echo "********************************************************************"
		@echo "    Launch TRACE"
		@echo "********************************************************************"
		trce -f $(TRACEOPTS) $(PROJNAME).ncd $(PROJNAME).pcf -o $(PROJNAME).twr
		cp $(PROJNAME).twr $(LOGDIR)/$(PROJNAME).twr

sdf : $(PROJNAME).sdf

$(PROJNAME).sdf : $(PROJNAME).ncd
		@echo "********************************************************************"
		@echo "    Launch NETGEN"
		@echo "********************************************************************"
		netgen -f $(SDFOPTS) $(PROJNAME).ncd
		cp $(PROJNAME).v $(CODEDIR)/$(PROJNAME)/sim/gate_sim/bench/$(PROJNAME).v
		cp $(PROJNAME).tv $(CODEDIR)/$(PROJNAME)/sim/gate_sim/bench/$(PROJNAME).tv
		cp $(PROJNAME).sdf $(CODEDIR)/$(PROJNAME)/sim/gate_sim/bench/$(PROJNAME).sdf
		mv $(PROJNAME).v $(OUTDIR)/
		mv $(PROJNAME).tv $(OUTDIR)/
		mv $(PROJNAME).nlf $(LOGDIR)/

download: $(PROJNAME).bit
		@echo "********************************************************************"
		@echo "    Download MCS File to PROM and Bitfile to FPGA"
		@echo "********************************************************************"
		impact -batch $(BINDIR)/download.cmd

all : synthesize translate map par trace bit prom
		@echo "********************************************************************"
		@echo "    This build has finished."
		@echo "********************************************************************"

clean : 
	rm -f $(PROJNAME)* 
	rm -f map.mrp map.ncd map.ngm netlist.lst
	rm -f *.log
